<?php

/* Usage
Grab some XML data, either from a file, URL, etc. however you want. Assume storage in $strYourXML;

$xml = new xml2Array($strYourXML);
xml array is in $xml->data;
    This is basically an array version of the XML data (no attributes), striaght-up. If there are
    multiple items with the same name, they are split into a numeric sub-array, 
    eg, <items><item test="123">foo</item><item>bar</item></items>
    becomes: array('item' => array(0=>array('item'=>'foo'), 1=>array('item'=>'foo'))
attributes are in $xml->attributes;
    These are stored with xpath type paths, as $xml->attributes['/items/item/0']["test"] == "123"
    

Other way (still works, but not as nice):

$objXML = new xml2Array();
$arrOutput = $objXML->parse($strYourXML);
print_r($arrOutput); //print it out, or do whatever!

*/

class xml2Array {
    var $arrOutput = array();
    var $resParser;
    var $strXmlData;
    
    var $attributes;
    var $data;
    
    function xml2Array($strInputXML = false) {
        if (!empty($strInputXML)) {
            $this->data = $this->parseAdvanced($strInputXML);
        }
    }
    
    function parse($strInputXML) {
    
            $this->resParser = xml_parser_create ();
            xml_set_object($this->resParser,$this);
            xml_set_element_handler($this->resParser, "tagOpen", "tagClosed");
            
            xml_set_character_data_handler($this->resParser, "tagData");
        
            $this->strXmlData = xml_parse($this->resParser,$strInputXML );
            if(!$this->strXmlData) {
                trigger_error(sprintf("XML error: %s at line %d",
                xml_error_string(xml_get_error_code($this->resParser)),
                xml_get_current_line_number($this->resParser)));
                return;
            }
                            
            xml_parser_free($this->resParser);
            
            return $this->arrOutput;
    }
    function tagOpen($parser, $name, $attrs) {
        $tag=array("name"=>$name,"attrs"=>$attrs); 
        @array_push($this->arrOutput,$tag);
    }
    
    function tagData($parser, $tagData) {
        if(trim($tagData)) {
            if(isset($this->arrOutput[count($this->arrOutput)-1]['tagData'])) {
                $this->arrOutput[count($this->arrOutput)-1]['tagData'] .= "\n".$tagData;
            } 
            else {
                $this->arrOutput[count($this->arrOutput)-1]['tagData'] = $tagData;
            }
        }
    }
    
    function tagClosed($parser, $name) {
            if(isset($this->arrOutput[count($this->arrOutput)-2])) {
        @$this->arrOutput[count($this->arrOutput)-2]['children'][] = $this->arrOutput[count($this->arrOutput)-1];
        array_pop($this->arrOutput);
            }
    }
    
    function recursive_parseLevel($items, &$attrs, $path = "") {
        $array = array();
        foreach (array_keys($items) as $idx) {
            @$items[$idx]['name'] = strtolower($items[$idx]['name']);
            
            $multi = false;
            if (isset($array[ $items[$idx]['name'] ])) {
                // this child is already set, so we're adding multiple items to an array 
                
                if (!is_array($array[ $items[$idx]['name'] ]) || !isset($array[ $items[$idx]['name'] ][0])) {
                    // hasn't already been made into a numerically-indexed array, so do that now
                    // we're basically moving the current contents of this item into a 1-item array (at the 
                    // original location) so that we can add a second item in the code below
                    $array[ $items[$idx]['name'] ] = array( $array[ $items[$idx]['name'] ] );

                    if (isset($attrs[ $path.'/'.$items[$idx]['name'] ])) {
                        // move the attributes to /0
                        $attrs[ $path.'/'.$items[$idx]['name'].'/0' ] = $attrs[ $path.'/'.$items[$idx]['name'] ];
                        unset($attrs[ $path.'/'.$items[$idx]['name'] ]);
                    }
                }
                $multi = true;
            }
            
            if ($multi) {    
                $newitem = &$array[ $items[$idx]['name'] ][];
            } else {
                $newitem = &$array[ $items[$idx]['name'] ];
            }
            
            
            if (isset($items[$idx]['children']) && is_array($items[$idx]['children'])) {
                $newitem = $this->recursive_parseLevel($items[$idx]['children'], $attrs, $path.'/'.$items[$idx]['name']);
            } else if (isset($items[$idx]['tagData'])) {
                $newitem = $items[$idx]['tagData'];
            } else {
                $newitem = false;
            }
            
            if (isset($items[$idx]['attrs']) && is_array($items[$idx]['attrs']) && count($items[$idx]['attrs'])) {
                $attrpath = $path.'/'.$items[$idx]['name'];
                if ($multi) {
                    $attrpath .= '/'.(count($array[ $items[$idx]['name'] ])-1);
                }
                foreach ($items[$idx]['attrs'] as $name=>$value) {
                    $attrs[ $attrpath ][ strtolower($name) ] = $value;
                }
            }
        }
        return $array;
    }
    
    function parseAdvanced($strInputXML) {
        $array = $this->parse($strInputXML);
        if(!is_array($array)) { return; }
        $this->attributes = array();
        return $this->data = $this->recursive_parseLevel($array, $this->attributes);
    }
}

